#include <cstdio>
#include <unistd.h>

#include "charsets.hh"

int main(int argc, const char *const *argv)
{
    bool ishtml = false;
    
    if(argv[1] && !strcmp(argv[1], "-h"))
    {
    	ishtml = true;
    	--argc, ++argv;
    }
    
    if(argc != 3)
    {
        static const char *const convlist[] = {
#define CHARCONV_LIST
#include "unicode/converters.hh"
#undef CHARCONV_LIST
            NULL
        };

        printf(
            "charconv v"VERSION" - Copyright (C) 1992,2001 Bisqwit (http://iki.fi/bisqwit/\n"
            "\n"
            "This is free software; see the source for copying conditions. There is no\n"
            "warranty; not even for merchantability or fitness for a particular purpose.\n"
            "\n"
            "Usage: charconv [-h] <incharset> <outcharset>\n"
            "\n"
            "Reads stdin, outputs stdout. Does incharset->outcharset conversion via unicode.\n"
            "\n"
            "-h = Input is html (THIS BUGS)\n"
            "Available character sets/encodings:\n");
        const char *const *s = convlist;
        int c=0;
        while(*s)
        {
            int width = printf("- %s", *s++);
            c++;
            if(c < 3 || ((c-3)%4)==0)printf("\n");
            else
            {
                printf("%*s", 18-width, "");
            }
        }
        printf("\n\n"
        	"Typoes are allowed to some degree in the character set names,\n"
        	"and some general aliases like latin* and iso* are known.\n"
        	"\n"
        	"Too much information? Pipe to a pager.\n");
        return 0;
    }
    
    charset *conv1 = newconverter(argv[1]);
    charset *conv2 = newconverter(argv[2]);
    
    for(;;)
    {
        char Buf[128]; /* Short strings are faster than long strings */
        
        int count = read(0, Buf, sizeof Buf);
        if(count <= 0) break;
        
        string result = conv2->uni2c(conv1->c2uni(string(Buf, count), ishtml));
        
        if(write(1, result.data(), result.size()) <= 0)
            break;
    }
    
    return 0;
}
