#include "eucjp.hh"

#include "unicode/jis0201.h"
#include "unicode/jis0208.h"
#include "unicode/jis0212.h"

eucjp::eucjp() : cache(0), instate(ascii)
{
	jis201in = new jis201_to_uni;
	jis201out= new uni_to_jis201;
	jis208in = new jis208_to_uni;
	jis208out= new uni_to_jis208;
	jis212in = new jis212_to_uni;
	jis212out= new uni_to_jis212;
}
eucjp::~eucjp()
{
	delete jis201in; delete jis201out;
	delete jis208in; delete jis208out;
	delete jis212in; delete jis212out;
}

const wstring eucjp::c2uni(const string &s, bool html)
{
    wstring ss;
    unsigned a, b=s.size();
    charmap::const_iterator i;
    for(a=0; a<b; ++a)
    {
        switch(instate)
        {
            case ascii:
                if(html)
                {
                    ss += TryParseEntities(s, a);
                    if(a==b)break;
                }
                if((unsigned char)s[a] < 0x80) ss += s[a];
                else if((unsigned char)s[a] == eucjpSS2) instate = ss2;
                else if((unsigned char)s[a] == eucjpSS3) instate = ss3a;
                else { instate = st208; cache = (unsigned char)s[a]; }
                break;
            case ss2:
                instate = ascii;
                i = jis201in->find((unsigned char)s[a]);
                if(i == jis201in->end())
                    ss += 0xFFFD; /* undefined char */
                else
                    ss += i->second;
                break;
            case ss3a:
                instate = ss3b;
                cache = s[a];
                break;
            case ss3b:
                instate = ascii;
                cache = cache*256 + (unsigned char)s[a];
                i = jis212in->find(cache & 0x7F7F);
                if(i == jis212in->end())
                    ss += 0xFFFD; /* undefined char */
                else
                    ss += i->second;
                break;
            case st208:
                instate = ascii;
                cache = cache*256 + (unsigned char)s[a];
                i = jis208in->find(cache & 0x7F7F);
                if(i == jis208in->end())
                    ss += 0xFFFD; /* undefined char */
                else
                    ss += i->second;
                break;
        }
    }
    return ss;
}

const string eucjp::uni2c(const wstring &s)
{
    string ss;
    unsigned a, b=s.size();
    for(a=0; a<b; ++a)
    {
        if(s[a] < 0x80)
        {
            /* ascii */
            ss += s[a];
            continue;
        }
        charmap::const_iterator i;
        
        /* jis201? */
        i = jis201out->find(s[a]);
        if(i != jis201out->end())
        {
            if(i->second < 0x80)
            {
                /* jis201 latin */
                ss += (char)i->second;
            }
            else
            {
                /* jis201 kana */
                ss += (char)eucjpSS2;
                ss += (char)(i->second);
            }
            continue;
        }

        /* jis208? */
        i = jis208out->find(s[a]);
        if(i != jis208out->end())
        {
            ss += (char)((i->second >> 8) | 0x80);
            ss += (char)((i->second & 255)| 0x80);
            continue;
        }

        /* jis212? */
        i = jis212out->find(s[a]);
        if(i != jis212out->end())
        {
            ss += (char)eucjpSS3;
            ss += (char)((i->second >> 8) | 0x80);
            ss += (char)((i->second & 255)| 0x80);
            continue;
        }
        
        if(s[a] == 0xA0)
            ss += ' ';
        else
            ss += '?';
    }
    return ss;
}
