#include <cstdio>
#include <cctype>
#include <algorithm>

using std::sprintf;
using std::isprint;
using std::sort;

#include "unihtml.hh"

static const struct { const char *ent; wchar_t ch; } EntityTable[] =
{
#include "entities.h"
};

#define ENTITYCOUNT (sizeof(EntityTable) / sizeof(EntityTable[0]))

class revsorter
{
public:
    bool operator() (unsigned a, unsigned b) const
    {
        return EntityTable[a].ch < EntityTable[b].ch;
    }
};
static class RevTable
{
public:
    unsigned key[ENTITYCOUNT];
    RevTable()
    {
        for(unsigned a=0; a<ENTITYCOUNT; ++a)key[a] = a;
        sort(&key[0], &key[ENTITYCOUNT], revsorter());
    }
} ReverseEntityTable;

const wchar_t FindEntity(const string &s)
{
    unsigned size = ENTITYCOUNT;
    unsigned eka=0, vika=size-1;
    for(;;)
    {
        unsigned n = (eka+vika)/2;
        if(s == EntityTable[n].ent)return EntityTable[n].ch;
        if(eka >= vika)break;
        if(s > EntityTable[n].ent) { eka=n+1; continue; }
        if(s < EntityTable[n].ent) { vika=n; continue; }
    }
    return '?';
}

const char *RevFindEntity(const wchar_t ch)
{
    unsigned size = ENTITYCOUNT;
    unsigned eka=0, vika=size-1;
    for(;;)
    {
        unsigned n = (eka+vika)/2;
        unsigned K = ReverseEntityTable.key[n];
        if(ch == EntityTable[K].ch)return EntityTable[K].ent;
        if(eka >= vika)break;
        if(ch > EntityTable[K].ch) { eka=n+1; continue; }
        if(ch < EntityTable[K].ch) { vika=n; continue; }
    }
    return NULL;
}

wstring TryParseEntities(const string &s, unsigned &a)
{
    wstring ss;
    while(a < s.size() && s[a]=='&')
    {
        ++a;
        unsigned c = s.find(';', a);
        if(c != s.npos)  
        {
            string code = s.substr(a, c-a);
            a = c+1;
            wchar_t ch = '?';
            if(code.size() > 0)
            {
                if(code[0] == '#')
                {
                    if(code.size() > 1 && (code[1] == 'x' || code[1] == 'X'))
                        ch = strtol(code.c_str()+2, NULL, 16);
                    else
                        ch = strtol(code.c_str()+1, NULL, 10);
                }
                else
                {
                    ch = FindEntity(code);
                }
            }
            ss += ch;
        }
    }
    return ss;
}

const wstring unihtml::c2uni(const string &s, bool html)
{
    wstring ss;
    unsigned a, b=s.size();
    for(a=0; a<b; ++a)
    {
    	/* FIXME: if(html), this should be entity-parsed *twice* */
        if(true || html)
        {
            ss += TryParseEntities(s, a);
            if(a==b)break;
        }
        ss += (unsigned char)s[a];
    }
    return ss;
}

const string unihtml::uni2c(const wstring &s)
{
    string ss;
    unsigned a, b=s.size();
    for(a=0; a<b; ++a)
    {
        const char *sss = RevFindEntity(s[a]);
        if(sss)
        {
            ss += '&';
            ss += sss;
            ss += ';';
            continue;
        }
        
        if(s[a] >= 0x20 && s[a] <= 0xFF && isprint(s[a])
        || s[a] == '\n')
        {
            ss += (char)s[a];
            continue;
        }
        
        char Buf[16];
        sprintf(Buf, "&#%u;", (unsigned)s[a]);
        ss += Buf;
    }
    return ss;
}
